/*
 * Copyright (C) 2012 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE COMPUTER, INC. ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL APPLE COMPUTER, INC. OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef InbandTextTrack_h
#define InbandTextTrack_h

#if ENABLE(VIDEO_TRACK)

#include "InbandTextTrackPrivate.h"
#include "InbandTextTrackPrivateClient.h"
#include "TextTrack.h"
#include <wtf/RefPtr.h>

namespace WebCore {

class Document;
class InbandTextTrackPrivate;
class MediaPlayer;
class TextTrackCue;

class InbandTextTrack : public TextTrack, public InbandTextTrackPrivateClient {
public:
    static PassRefPtr<InbandTextTrack> create(ScriptExecutionContext*, TextTrackClient*, PassRefPtr<InbandTextTrackPrivate>);
    virtual ~InbandTextTrack();

    virtual void setMode(const AtomicString&) OVERRIDE;
    size_t inbandTrackIndex();

    void setPrivate(PassRefPtr<InbandTextTrackPrivate>);

    virtual bool isInband() const OVERRIDE { return true; }

private:
    InbandTextTrack(ScriptExecutionContext*, TextTrackClient*, PassRefPtr<InbandTextTrackPrivate>);

    void setModeInternal(const AtomicString&);
    void updateKindFromPrivate();

    virtual void addCue(InbandTextTrackPrivate*, double, double, const String&, const String&, const String&) OVERRIDE;

    virtual void willRemoveTextTrackPrivate(InbandTextTrackPrivate*) OVERRIDE;

#if USE(PLATFORM_TEXT_TRACK_MENU)
    virtual InbandTextTrackPrivate* privateTrack() OVERRIDE { return m_private.get(); }
#endif

    RefPtr<InbandTextTrackPrivate> m_private;
};

#define TYPE_CASTS_BASE(ToClassName, argumentType, argumentName, pointerPredicate, referencePredicate) \
inline ToClassName* to##ToClassName(argumentType* argumentName) \
{ \
    ASSERT(!argumentName || (pointerPredicate)); \
    return static_cast<ToClassName*>(argumentName); \
} \
inline const ToClassName* to##ToClassName(const argumentType* argumentName) \
{ \
    ASSERT(!argumentName || (pointerPredicate)); \
    return static_cast<const ToClassName*>(argumentName); \
} \
inline ToClassName& to##ToClassName(argumentType& argumentName) \
{ \
    ASSERT(referencePredicate); \
    return static_cast<ToClassName&>(argumentName); \
} \
inline const ToClassName& to##ToClassName(const argumentType& argumentName) \
{ \
    ASSERT(referencePredicate); \
    return static_cast<const ToClassName&>(argumentName); \
} \
void to##ToClassName(const ToClassName*); \
void to##ToClassName(const ToClassName&);

TYPE_CASTS_BASE(InbandTextTrack, TextTrack, track, track->isInband(), track.isInband());

#undef TYPE_CASTS_BASE

} // namespace WebCore

#endif
#endif
